#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "../decode.h"

#define MAX_BUFFER_LEN 1024

int main(int argc,char **argv)
{
    char data[MAX_BUFFER_LEN],s[MAX_BUFFER_LEN],
         stat_len;  /* this is the length of a single statement */
    unsigned char f2c,verb=0;
    int i,errors=0,punta=0,len;
    FILE *input_file;
    struct stat buf;

    if ((argc!=2)&&(argc!=3))
    {
        printf("strencode encodes a text file in order to use it with mp3guessenc.\nUsage:\tstrencode [-v] filename\n\n");
        return 0;
    }

/*
 * Now I'm sure argc is 2 (verbose option was not set) or 3 (verbose option was set)
 */

    if (argc==3)
    {
        if ((argv[1][0]=='-')&&(argv[1][1]=='v'))
            verb=1; /* verbose mode was asked */
        else
        {
            printf("The only valid option is `-v' (verbose).\n");
            return 1;
        }
    }

    /* set position of filename to check */
    if (argc==2)
        f2c=1;
    else
        f2c=2;

    if (stat(argv[f2c],&buf)==-1)
    {
        printf("Error retrieving informations about `%s', exiting.\n",argv[f2c]);
        return 1;
    }

    if (!S_ISREG(buf.st_mode))
    {
        printf("`%s' not a regular file, exiting.\n",argv[f2c]);
        return 1;
    }

    input_file=fopen(argv[f2c],"rb");
    if (input_file==NULL)
    {
        printf("Error opening in read-only `%s', exiting.\n",argv[f2c]);
        return 2;
    }

    if (verb) printf("Reading the file.\n");
    i=fgetc(input_file);

    while (!feof(input_file)&&(punta!=MAX_BUFFER_LEN-1))
    {
//printf("punta=%u, i=%d\n",punta,i);
        data[punta++]=i;
        i=fgetc(input_file);
    }
    fclose(input_file);
//printf("data[%u]=0;\n",punta);
    data[punta]=0;

    if (verb)
        printf("Stop reading `%s' (file length %d b), ", argv[f2c],(int)buf.st_size);

    if (punta==MAX_BUFFER_LEN-1)
        printf("WARNING: buffer is full!\n");
    else
        if (verb) printf("all went OK.\n");

    len=strlen(data);
#define MAX_STATEMENT 77
    for (i=0; i<len+1; i++) s[i]=data[i];/* s[len]=0;*/
    if (verb) printf("string:\n%s\nlength=%d\n",s,len);
    len++; /* I need this in order to include the string terminator `\0' in the scramble */

/*
 * NOW I'VE GOT MY DATA INTO `data' AND INTO `s', I WILL NOW DO 4 MAIN STEPS:
 * 1. SCRAMBLE DATA `s' VIA `decode'
 * 2. PRINT SCRAMBLED STRING
 * 3. SCRAMBLE DATA `s' AGAIN
 * 4. COMPARE DATA WITH THE UN-MODIFIED COPY INTO `data'
 */

    decode (s,len);
    if (verb) printf("scramble done!\nthe new string is:\n");
    for (i=0; i<len; i++)
        if (verb)
            {if (isprint(s[i])) printf("%c",s[i]); else printf(".");}
    if (verb) printf("\n");

    printf("\n#define DATA_LEN %u\n",len);
    stat_len=printf("char scrambled_data[DATA_LEN]={");
    for (i=0; i<len-1; i++)
    {
      stat_len+=printf("%u,",(unsigned char)s[i]);
      if (stat_len>MAX_STATEMENT)
      {
        printf("\n");
        stat_len=0;
      }
    }
    printf("%u};\n\n",(unsigned char)s[len-1]);

    decode (s,len);
    if (verb)
    {
        printf("scramble done again!\nstring:\n%s\nlength=%d\n",s,(int)strlen(s));
        printf("\nComparison...\n");
    }
    for (i=0; i<len; i++) if (s[i]!=data[i]) {if (verb) printf("ERROR AT CHAR %u!!!\n",i); errors++;}
    if (!errors)
    {
        if (verb) printf("No errors detected.\n");
        return 0;
    }
    else
    {
        if (verb) printf ("%u errors detected.\n",errors);
        return 3;
    }
}
